(*
  Funzionalit di firma e marcatura temporale file utilizzando il token di
  firma digitale.

  IMPORTANTE: la firma ottenuta con la funzione FirmaFile()  di tipo CAdES.

              Se si volesse anche di tipo PaDes si devono aggiungere alcune
              propriet (PDF Reason, PDF Name e PDF Location), bisogna impostare
              il tipo di file in input e si deve impostare il nome di file di
              output.

  NOTA: E' necessario installare il RunTime di Visual Studio 2012
*)

unit QDigitSign;

interface

uses
  Classes;

const
   DLL_NAME_LIBRARY_QDIGITSIGN = 'QDigitSign.dll';               // Nome della DLL
   LOG_NAME_LIBRARY_QDIGITSIGN = 'QDigitSign.log';               // Nome del LOG della DLL

   // Nomi delle funzioni e delle procedure esposte
   FUNCT_NAME_QDS_DGTSIGN_SET          = 'qdigitsign_set';
   PROC_NAME_QDS_DGTSIGN_CLEANUP       = 'qdigitsign_cleanup';

   // Firma
   FUNCT_NAME_QDS_DGTSIGN_SIGN_INIT    = 'qdigitsign_sign_init';
   FUNCT_NAME_QDS_DGTSIGN_SIGN_SET     = 'qdigitsign_sign_set';
   FUNCT_NAME_QDS_DGTSIGN_SIGN_SIGN    = 'qdigitsign_sign_sign';
   FUNCT_NAME_QDS_DGTSIGN_SIGN_CLEANUP = 'qdigitsign_sign_cleanup';

   // Opzioni generali
   QDIGITSIGN_OPT_PKCS11                    = 1;
   QDIGITSIGN_OPT_SLOT                      = 2;
   QDIGITSIGN_OPT_PIN                       = 3;
   QDIGITSIGN_OPT_ALIAS                     = 4;
   QDIGITSIGN_OPT_CADES                     = 5;
   QDIGITSIGN_OPT_XADES                     = 5;
   QDIGITSIGN_OPT_DETACHED                  = 6;
   QDIGITSIGN_OPT_INPUTFILE                 = 7;
   QDIGITSIGN_OPT_OUTPUTFILE                = 8;
   QDIGITSIGN_OPT_INPUTFILE_TYPE            = 9;
   QDIGITSIGN_OPT_TSA_URL                   = 10;
   QDIGITSIGN_OPT_TSA_USERNAME              = 11;
   QDIGITSIGN_OPT_TSA_PASSWORD              = 12;
   QDIGITSIGN_OPT_VERIFY_REVOCATION         = 13;
   QDIGITSIGN_OPT_LOG_LEVEL                 = 14;
   QDIGITSIGN_OPT_LOG_FILE                  = 15;
   QDIGITSIGN_OPT_INPUTFILE_PLAINTEXT       = 16;
   QDIGITSIGN_OPT_CACERT_DIR                = 17;
   QDIGITSIGN_OPT_PDF_SUBFILTER             = 18;
   QDIGITSIGN_OPT_CONFIG_FILE               = 19;
   QDIGITSIGN_OPT_PROXY                     = 20;
   QDIGITSIGN_OPT_PROXY_PORT                = 21;
   QDIGITSIGN_OPT_PROXY_USRPASS             = 22;
   QDIGITSIGN_OPT_OID_MAP_FILE              = 23;
   QDIGITSIGN_OPT_TCP_TIMEOUT               = 24;
   QDIGITSIGN_OPT_PDF_REASON                = 25;
   QDIGITSIGN_OPT_PDF_NAME                  = 26;
   QDIGITSIGN_OPT_PDF_LOCATION              = 27;
   QDIGITSIGN_OPT_PDF_PAGE	                 = 28;
   QDIGITSIGN_OPT_PDF_LEFT				        = 29;
   QDIGITSIGN_OPT_PDF_BOTTOM			        = 30;
   QDIGITSIGN_OPT_PDF_WIDTH			        = 31;
   QDIGITSIGN_OPT_PDF_HEIGHT			        = 32;
   QDIGITSIGN_OPT_PDF_IMAGEPATH			     = 33;
   QDIGITSIGN_OPT_PDF_GRAPHOMETRIC_DATA	  = 34;
   QDIGITSIGN_OPT_PDF_GRAPHOMETRIC_DATA_VER = 35;
   QDIGITSIGN_OPT_ATR_LIST_FILE		        = 36;
   QDIGITSIGN_OPT_HASH_ALGO			        = 37;
   QDIGITSIGN_OPT_LICENSEE				        = 38;
   QDIGITSIGN_OPT_PRODUCTKEY			        = 39;
   QDIGITSIGN_OPT_RS_OTP_PIN			        = 40;
   QDIGITSIGN_OPT_RS_HSMTYPE			        = 41;
   QDIGITSIGN_OPT_RS_TYPE_OTP_AUTH          = 42;
   QDIGITSIGN_OPT_RS_USERNAME			        = 43;
   QDIGITSIGN_OPT_RS_PASSWORD			        = 44;
   QDIGITSIGN_OPT_RS_CERTID			        = 45;
   QDIGITSIGN_OPT_RS_SERVICE_URL		        = 46;

   QDIGITSIGN_OPT_PDF_DESCRIPTION		        = 50;
   QDIGITSIGN_OPT_PDF_NAME_LABEL		= 51;
   QDIGITSIGN_OPT_PDF_REASON_LABEL		= 52;
   QDIGITSIGN_OPT_PDF_LOCATION_LABEL	= 53;


   DGTSGN_NO_ERROR = 0;

   // Livelli di LOG
   LOG_TYPE_ERROR   = 1;
   LOG_TYPE_WARNING = 2;
   LOG_TYPE_MESSAGE = 3;
   LOG_TYPE_DEBUG   = 4;

   // Tipo di file
   QDIGITSIGN_FILETYPE_PLAINTEXT = 0;
   QDIGITSIGN_FILETYPE_P7M       = 1;
   QDIGITSIGN_FILETYPE_PDF       = 2;
   QDIGITSIGN_FILETYPE_M7M       = 3;
   QDIGITSIGN_FILETYPE_TSR       = 4;
   QDIGITSIGN_FILETYPE_TST       = 5;
   QDIGITSIGN_FILETYPE_TSD       = 6;
   QDIGITSIGN_FILETYPE_XML       = 7;
   QDIGITSIGN_FILETYPE_AUTO      = 8;


   // Valori da utilizzare per la firma TUTTI RELATIVI al TOKEN della Bit4Id di tipo Key4
   QDIGITSIGN_LICENSEE    = 'lllll';                                 // QDigitSign: licenziatario della dll
   QDIGITSIGN_PRODUCTKEY  = 'ppppppppp';                             // QDigitSign: chiave del prodotto

   NOME_MODULO_PKCS11 = 'bit4ipki.dll';                                // Nome della dll PKCS#11 tipica del token utilizzato
   SLOT_TOKEN_DEFAULT = 0;                                             // Primo token
   FLAG_CADES         = 1;                                             // generazione CAdES (0=no CAdES; 1=CAdES)
   PIN_TOKEN          = '00000000';                                    // PIN per accedere al Token
   ALIAS_CERTIFICATO  = 'DS User Certificate0';                        // Alias del certificato da usare per la firma
   TSA_URL            = 'https://marte.infocert.it/cdie/HttpService';  // URL del TSA InfoCert (fornito da InfoCert il 16 Settembre 2013)
   TSA_USERNAME       = 'uuuuu@nnnnn.it';                              // Username per accedere al TSA
   TSA_PASSWORD       = 'pppppppppppppp';                              // Password per accedere al TSA
   TIPO_FILE_2SIGN    = QDIGITSIGN_FILETYPE_PLAINTEXT;                 // tipo di file da firmare
   FLAG_VERIFICA_CRL  = 0;                                             // verifica della CRL (0=no; 1=si)
   FLAG_FIRMADETACHED = 0;                                             // generazione di firma detached (0=no; 1=si)

   EXTENSION_P7M = '.P7M';
   EXTENSION_M7M = '.M7M';

//   EXTENSION_PDF = '.PDF';

type
   QDIGITSIGN_CTX = Pointer;

   TQDigitSign_Set          = function (Option: Integer; value: Pointer): Longint; stdcall;
   TQDigitSign_CleanUp      = procedure ; stdcall;
   // Firma
   TQDigitSign_Sign_Init    = function: QDIGITSIGN_CTX; stdcall;
   TQDigitSign_Sign_Set     = function (ctx: QDIGITSIGN_CTX; Option: Integer; value: Pointer): Longint; stdcall;
   TQDigitSign_Sign_Sign    = function (ctx: QDIGITSIGN_CTX): Longint; stdcall;
   TQDigitSign_Sign_CleanUp = function (ctx: QDIGITSIGN_CTX): Longint; stdcall;


   TQDigitSignLib = class(TObject)
   private
     HLibrary: THandle;             // Handle della DLL

     function  FirmaFile(aFileName: String; aWithLog, aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;
     function  GoFirmaFile(aFileName: String; aWithLOG, aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;

   public
     QDigitSign_Set:     TQDigitSign_Set;
     QDigitSign_CleanUp: TQDigitSign_CleanUp;
     // Firma
     QDigitSign_Sign_Init:    TQDigitSign_Sign_Init;
     QDigitSign_Sign_Set:     TQDigitSign_Sign_Set;
     QDigitSign_Sign_Sign:    TQDigitSign_Sign_Sign;
     QDigitSign_Sign_CleanUp: TQDigitSign_Sign_CleanUp;

     class function Firma_MarcaFile(aFileName: String; aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;

     // Le lascio fra le pubbliche per poterle richiamare direttamente ---------
     function  LoadDynamicLib: Boolean;
     procedure FreeDynamicLib;
     // Le lascio fra le pubbliche per poterle richiamare direttamente ---------

   end;



implementation


uses
  Windows, SysUtils;

// -----------------------------------------------------------------------------
// -- Funzioni locali ----------------------------------------------------------
// -----------------------------------------------------------------------------

function FilePathDLL: string;
// -----------------------------------------------------------------------------
// Restituisce il filename della DLL (compreso il Path completo)
var
   aBuffer: array [0..MAX_PATH-1] of WideChar;
begin
   SetString(Result, aBuffer, GetModuleFileNameW(hInstance, aBuffer, SizeOf(aBuffer)));
end;



function GetFileNameLOG(aFileName: String): String;
// -----------------------------------------------------------------------------
// Restituisce il nome del file di LOG per il file in input apponendo alla fine
// del nome del file in input l'estensione .LOG.
// NOTA: il nome di file restituito  univoco in quella directory ed  del tipo
//                         <aFileName>.LOG;
// se ne dovesse gi esistere qualcuno, il nome restituito sar del tipo:
//                         <aFileName>_n.LOG,
// dove n  un progressivo numerico utilizzato per differenziare i vari file.
const
   EXTENSION_LOG = '.LOG';
var
  xNomeLOG:     String;
  xSuffissoNum: Integer;
begin
   // Nome del file di LOG
   xNomeLOG := aFileName + EXTENSION_LOG;

   xSuffissoNum := 1;
   while True do begin
      if not(FileExists(xNomeLOG)) then Break;

      // Un file con il nome xNomeLOG gi esiste: genero un nuovo nome per il file di LOG
      xNomeLOG := aFileName + '_' + IntToStr(xSuffissoNum) + EXTENSION_LOG;
      Inc(xSuffissoNum);
   end;

   Result := xNomeLOG;
end;



function GetNomeFileFinale(aFileName: String; aIsP7M_M7MExtension, aWithMarcaTemporale: Boolean): String;
// -----------------------------------------------------------------------------
// Restituisce:
//  aFileName + .p7m,                             se aIsP7MExtension=1 e aWithMarcaTemporale=0
//  aFileName + .m7m,                             se aIsP7MExtension=1 e aWithMarcaTemporale=1
//  aFileName + .p7m.<estensione file originale>, se aIsP7MExtension=0 e aWithMarcaTemporale=0
//  aFileName + .m7m.<estensione file originale>, se aIsP7MExtension=0 e aWithMarcaTemporale=1
var
   xEstensioneORIG: String;
begin
   Result := aFileName;

   if aIsP7M_M7MExtension then begin
      if aWithMarcaTemporale
         then Result := Result + EXTENSION_M7M
         else Result := Result + EXTENSION_P7M;
      end
   else begin
      xEstensioneORIG := ExtractFileExt(Result);

      if aWithMarcaTemporale
         then Result := ChangeFileExt(Result, EXTENSION_M7M)
         else Result := ChangeFileExt(Result, EXTENSION_P7M);

      Result := Result + xEstensioneORIG;
   end;
end;



// -----------------------------------------------------------------------------
// -- Funzioni Private ---------------------------------------------------------
// -----------------------------------------------------------------------------

function TQDigitSignLib.FirmaFile(aFileName: String; aWithLog, aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;
// -----------------------------------------------------------------------------
// Esegue un tentativo di Firma.
var
   xRes:   Boolean;
begin
   Result := True;
   if not(LoadDynamicLib) then begin
      aErrStr := 'Caricamente dinamico DLL non riuscito!';
      Result := False;
      Exit;
   end;

   try
      Result := GoFirmaFile(aFileName, aWithLog, aWithMarcaTemporale, aIsP7MExtension, aErrStr);
   finally
      // Scarico la DLL caricata dinamicamente in precedenza
      FreeDynamicLib;
   end;
end;



function TQDigitSignLib.GoFirmaFile(aFileName: String; aWithLOG, aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;
// -----------------------------------------------------------------------------
// Esegue la firma del file in input.
var
   Ret,
   xSlotToken,
   xCades,
   xTipoFile,
   xVerificaCRL,
   xFirmaDetached:  Integer;
   ctx:             QDIGITSIGN_CTX;
   xQDSLicensee,
   xQDSProductKey,
   xLogFile,
   xPKCS11Lib,
   xPIN,
   xAlias,
   xFileIN,
   xFileOUT,
   xTSAUrl,
   xTSAUserName,
   xTSAPassword,
   xEstensioneORIG: AnsiString;
begin
   Result := False;
   try
      // Imposto Licensee della QDigitSign.dll
      xQDSLicensee := QDIGITSIGN_LICENSEE;
      Ret := QDigitSign_Set(QDIGITSIGN_OPT_LICENSEE, PAnsiChar(xQDSLicensee));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del Licensee (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto il ProductKey della QDigitSign.dll
      xQDSProductKey := QDIGITSIGN_PRODUCTKEY;
      Ret := QDigitSign_Set(QDIGITSIGN_OPT_PRODUCTKEY, PAnsiChar(xQDSProductKey));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del ProductKey (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      if aWithLOG then begin
         // Imposto il file di log (nella directory dove risiede il programma di test)
         // Il nome del file di LOG  lo stesso del file che deve essere firmato
         // con l'aggiunta dell'estensione .LOG
         xLogFile := GetFileNameLOG(aFileName);
         if xLogFile <> '' then begin
            Ret := QDigitSign_Set(QDIGITSIGN_OPT_LOG_FILE, PAnsiChar(xLogFile));
            if Ret <> DGTSGN_NO_ERROR then begin
               aErrStr := 'Errore nell''indicazione del file di LOG (' + IntToStr(Ret) + ')!';
               Exit;
            end;

            // Imposto il Livello di Log (livello di debugging)
            Ret := QDigitSign_Set(QDIGITSIGN_OPT_LOG_LEVEL, Pointer(LOG_TYPE_DEBUG{4}));
            if Ret <> DGTSGN_NO_ERROR then begin
               aErrStr := 'Errore nell''indicazione del livello di LOG (' + IntToStr(Ret) + ')!';
               Exit;
            end;
         end;
      end;

      // Inizializzo la funzione di firma
      ctx := nil;
      ctx := QDigitSign_Sign_Init;

      // Imposto il modulo PKCS#11 da usare nella firma (deve essere nella directory corrente)
      xPKCS11Lib := NOME_MODULO_PKCS11;
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_PKCS11, PAnsiChar(xPKCS11Lib));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del modulo PKCS#11 (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto lo slot nel quale  presente il token
      xSlotToken := SLOT_TOKEN_DEFAULT;
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_SLOT, Pointer(xSlotToken));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione dello SLOT in cui trovare il token (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto lopzione di generazione di firma CAdES
      xCades := Integer(FLAG_CADES);
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_CADES, Pointer(xCades));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione dell''opzione di generazione con/senza CAdES (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto il PIN del token
      xPIN := PIN_TOKEN;
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_PIN, PAnsiChar(xPIN));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del PIN (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto lalias del certificato da usare per la firma
      xAlias := ALIAS_CERTIFICATO;
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_ALIAS, PAnsiChar(xAlias));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione dell''Alias del Certificato (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Marcatura temporale (se richiesta) ------------------------------------
      if aWithMarcaTemporale then begin
         // Url del TSA
         xTSAUrl := TSA_URL;
         Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_TSA_URL, PAnsiChar(xTSAUrl));
         if Ret <> DGTSGN_NO_ERROR then begin
            aErrStr := 'Errore nell''indicazione dell''URL del TSA (' + IntToStr(Ret) + ')!';
            Exit;
         end;

         // UserName dell'account per accedere al TSA
         xTSAUserName := TSA_USERNAME;
         if xTSAUserName <> '' then begin
            Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_TSA_USERNAME, PAnsiChar(xTSAUserName));
            if Ret <> DGTSGN_NO_ERROR then begin
               aErrStr := 'Errore nell''indicazione dello Username del TSA (' + IntToStr(Ret) + ')!';
               Exit;
            end;

            // Password dell'account per accedere al TSA
            xTSAPassword := TSA_PASSWORD;
            Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_TSA_PASSWORD, PAnsiChar(xTSAPassword));
            if Ret <> DGTSGN_NO_ERROR then begin
               aErrStr := 'Errore nell''indicazione della Password (' + IntToStr(Ret) + ')!';
               Exit;
            end;
         end;
      end;
      // Marcatura temporale (se richiesta) ------------------------------------

      // Imposto il nome del file da firmare
      xFileIN := Trim(aFileName);
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_INPUTFILE, PAnsiChar(xFileIN));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del nome del file da firmare (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto il tipo di file da firmare
      xTipoFile := TIPO_FILE_2SIGN;
      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_INPUTFILE_TYPE, Pointer(xTipoFile));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del tipo del file da firmare (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto la verifica della CRL sul certificato del sottoscrittore prima di apporre la firma
      xVerificaCRL := FLAG_VERIFICA_CRL;
      Ret := QDigitSign_Sign_Set(ctx,  QDIGITSIGN_OPT_VERIFY_REVOCATION, Pointer(xVerificaCRL));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione della verifica del CRL (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto lopzione di generazione di firma detached.
      xFirmaDetached := FLAG_FIRMADETACHED;
      Ret := QDigitSign_Sign_Set(ctx,  QDIGITSIGN_OPT_DETACHED, Pointer(xFirmaDetached));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del''opzione di gereazione firma Detached (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Imposto il nome del file di output (il default  P7M: se richiesto, dopo
      // P7M viene apposta l'estensione del file originale. Se  richiesta la marcatura
      // temporale l'estensione di default  M7M).
      xFileOUT := GetNomeFileFinale(aFileName, aIsP7MExtension, aWithMarcaTemporale);

      Ret := QDigitSign_Sign_Set(ctx, QDIGITSIGN_OPT_OUTPUTFILE, PAnsiChar(xFileOUT));
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore nell''indicazione del''estensione ' + xEstensioneORIG + ' per l''output (' + IntToStr(Ret) + ')!';
         Exit;
      end;

      // Firmo il file in base alle indicazioni date
      Ret := QDigitSign_Sign_Sign(ctx);
      if Ret <> DGTSGN_NO_ERROR then begin
         aErrStr := 'Errore durante l''apposizione della firma (' + IntToStr(Ret) + ')!';
         end
      else begin
         Result := True;   // Firma avvenuta con successo!
      end;

      if ctx <> nil then begin
         // Libero la memoria allocata nell'operazione di firma
         try
         {Ret := }QDigitSign_Sign_CleanUp(ctx);
         except
            aErrStr := 'Errore durante QDigitSign_Sign_CleanUp!';
         end;
      end;
   finally
      try
         QDigitSign_CleanUp;
      except
         aErrStr := 'Errore durante QDigitSign_CleanUp!';
      end;
      if ctx <> nil then ctx := nil;
   end;
end;



// -----------------------------------------------------------------------------
// -- Funzioni Pubbliche -------------------------------------------------------
// -----------------------------------------------------------------------------

class function TQDigitSignLib.Firma_MarcaFile(aFileName: String; aWithMarcaTemporale, aIsP7MExtension: Boolean; var aErrStr: String): Boolean;
// -----------------------------------------------------------------------------
// Esegue la firma (con marcatura o meno) del file passato in input
var
   aObjDLL: TQDigitSignLib;
   xStr:    String;
begin
   Result  := True;
   aErrStr := '';

   // Carico dinamicamente la DLL QDigitSign
   aObjDLL := nil;
   aObjDLL := TQDigitSignLib.Create;

   try
      Result := aObjDLL.FirmaFile(aFileName, False, aWithMarcaTemporale, aIsP7MExtension, aErrStr);
   finally
      if (aObjDLL <> nil) then begin
         aObjDLL.Free;
         aObjDLL := nil;
      end;
   end;
end;



function TQDigitSignLib.LoadDynamicLib: Boolean;
// -----------------------------------------------------------------------------
// Controlla e carica la DLL: restituisce FALSE se non si riesce a caricare
// la DLL.
begin
   QDigitSign_Set          := nil;
   QDigitSign_CleanUp      := nil;
   QDigitSign_Sign_Init    := nil;
   QDigitSign_Sign_Set     := nil;
   QDigitSign_Sign_Sign    := nil;
   QDigitSign_Sign_CleanUp := nil;

   Result := (HLibrary <> 0); // TRUE se gi caricata
   if Result then Exit;

   HLibrary := LoadLibrary(PChar(ExtractFilePath(FilePathDLL) + DLL_NAME_LIBRARY_QDIGITSIGN));

   if HLibrary <> 0 then begin
      QDigitSign_Set          := GetProcAddress(HLibrary, FUNCT_NAME_QDS_DGTSIGN_SET);
      QDigitSign_CleanUp      := GetProcAddress(HLibrary, PROC_NAME_QDS_DGTSIGN_CLEANUP);
      // Firma
      QDigitSign_Sign_Init    := GetProcAddress(HLibrary, FUNCT_NAME_QDS_DGTSIGN_SIGN_INIT);
      QDigitSign_Sign_Set     := GetProcAddress(HLibrary, FUNCT_NAME_QDS_DGTSIGN_SIGN_SET);
      QDigitSign_Sign_Sign    := GetProcAddress(HLibrary, FUNCT_NAME_QDS_DGTSIGN_SIGN_SIGN);
      QDigitSign_Sign_CleanUp := GetProcAddress(HLibrary, FUNCT_NAME_QDS_DGTSIGN_SIGN_CLEANUP);
      Result := True;
   end;
end;



procedure TQDigitSignLib.FreeDynamicLib;
// -----------------------------------------------------------------------------
// Ripulisce i riferimenti alle funzioni/procedure e scarica la DLL dalla memoria
begin
   QDigitSign_Set          := nil;
   QDigitSign_CleanUp      := nil;
   QDigitSign_Sign_Init    := nil;
   QDigitSign_Sign_Set     := nil;
   QDigitSign_Sign_Sign    := nil;
   QDigitSign_Sign_CleanUp := nil;

   if HLibrary=0 then Exit;
   FreeLibrary(HLibrary);
   HLibrary     := 0;
end;



end.
